<?php

namespace Drupal\webform_scheduled_tasks_plus;

use League\Flysystem\Filesystem;
use League\Flysystem\FilesystemException;
use League\Flysystem\UnableToWriteFile;
use Psr\Log\LoggerInterface;

/**
 * Trait used for Flysystem library.
 */
trait FlysystemAdapterTrait {

  /**
   * Logging Service.
   *
   * @var \Psr\Log\LoggerInterface|null
   */
  protected ?LoggerInterface $logger = NULL;

  /**
   * Return logging service.
   *
   * @return \Psr\Log\LoggerInterface
   *   Logger Service.
   */
  protected function getLogger(): LoggerInterface {
    if (!$this->logger) {
      $this->logger = \Drupal::logger('webform_scheduled_tasks_plus');
    }
    return $this->logger;
  }

  /**
   * Return Filesystem to act on.
   *
   * @return \League\Flysystem\Filesystem
   *   Return the filesystem.
   */
  protected function getFileSystem(): Filesystem {
    if (!method_exists($this, 'getAdapter')) {
      throw new \Exception('You must implement method getAdapter()');
    }
    $adapter = $this->getAdapter();
    return new Filesystem($adapter);
  }

  /**
   * Transport to remote system.
   *
   * @param string $fileName
   *   The file name to transport.
   *
   * @return bool
   *   Return TRUE if the transport process was successful.
   */
  public function transport(string $fileName): bool {
    try {
      $fileSystem = $this->getFileSystem();
      $contents = file_get_contents($fileName);
      $baseFileName = basename($fileName);
      $fileSystem->write($baseFileName, $contents);
    }
    catch (FilesystemException | UnableToWriteFile $exception) {
      $this->getLogger()->error($exception->getMessage());
      return FALSE;
    }
    return TRUE;
  }

}
