<?php

namespace Drupal\webform_scheduled_tasks_plus;

use Drupal\Core\Cache\CacheBackendInterface;
use Drupal\Core\Extension\ModuleHandlerInterface;
use Drupal\Core\Plugin\CategorizingPluginManagerTrait;
use Drupal\Core\Plugin\DefaultPluginManager;
use Drupal\webform_scheduled_tasks_plus\Annotation\WebformScheduledTaskPlusTransportPlugin as WebformScheduledTaskPlusTransportPluginAnnotation;
use Drupal\webform_scheduled_tasks_plus\Attribute\WebformScheduledTaskPlusTransportPlugin as WebformScheduledTaskPlusTransportPluginAttribute;

/**
 * A plugin manager for transporter plugins.
 */
class WebformScheduledTasksTransportPluginManager extends DefaultPluginManager {

  use CategorizingPluginManagerTrait {
    getSortedDefinitions as traitGetSortedDefinitions;
  }

  /**
   * Creates the discovery object.
   *
   * @param \Traversable $namespaces
   *   An object that implements \Traversable which contains the root paths
   *   keyed by the corresponding namespace to look for plugin implementations.
   * @param \Drupal\Core\Cache\CacheBackendInterface $cache_backend
   *   Cache backend instance to use.
   * @param \Drupal\Core\Extension\ModuleHandlerInterface $module_handler
   *   The module handler to invoke the alter hook with.
   */
  public function __construct(\Traversable $namespaces, CacheBackendInterface $cache_backend, ModuleHandlerInterface $module_handler) {
    $subdir = 'Plugin/WebformScheduledTasks/TransportPlugin';
    $plugin_interface = WebformScheduledTasksTransportPluginInterface::class;
    $plugin_definition_attribute_name = WebformScheduledTaskPlusTransportPluginAttribute::class;
    $plugin_definition_annotation_name = WebformScheduledTaskPlusTransportPluginAnnotation::class;
    parent::__construct($subdir, $namespaces, $module_handler, $plugin_interface, $plugin_definition_attribute_name, $plugin_definition_annotation_name);
    $this->alterInfo('webform_scheduled_tasks_plus_transport_plugin_info');
    $this->setCacheBackend($cache_backend, 'webform_scheduled_tasks_plus_transport_plugin_info', ['webform_scheduled_tasks_plus_transport_plugin_info']);
  }

  /**
   * {@inheritdoc}
   */
  public function getOptions() {
    $plugin_definitions = $this->getDefinitions();
    $plugin_definitions = $this->getSortedDefinitions($plugin_definitions);
    $plugin_definitions = $this->removeExcludeDefinitions($plugin_definitions);

    $options = [];
    foreach ($plugin_definitions as $plugin_id => $plugin_definition) {
      $options[$plugin_id] = $plugin_definition['label'];
    }
    return $options;
  }

  /**
   * {@inheritdoc}
   */
  public function removeExcludeDefinitions(array $definitions) {
    return $definitions ?? $this->getDefinitions();
  }

  /**
   * {@inheritdoc}
   */
  public function getSortedDefinitions(array|null $definitions = NULL, $sort_by = 'label') {
    // Sort the plugins first by category, then by label.
    foreach ($definitions as &$definition) {
      $this->processDefinitionCategory($definition);
    }

    $definitions = $this->traitGetSortedDefinitions($definitions, $sort_by);
    return $definitions;
  }

}
